import click
from gumshoe.classes import habits
from gumshoe.models import models

@click.command()
@click.argument("name", nargs=-1, required=True)
@click.option("--quota", "-q", type=int, default=1, help="Amount of quota your wish to complete.")
def cli(name,quota):
    """The complete command adds a completion time and date for a specified habit. Specify the name of the habit you wish to updated."""

    name = ' '.join(name)
    db = models.HabitModel()
    db2 = models.ActivityModel()

    habit = db.check_habit_exists(habits.Habit(name, "", ""))
    if len(habit) == 0:
        habit_name = click.style("{0}".format(name), fg='red')
        click.echo("This habit '{0}' doe not exists, please use the create function to create it or use the show function "
               "to list all habits".format(habit_name))
    else:
        for x in range(quota):
            if not db2.complete_activity(habit[0]):
                click.echo("Seems there has been a system error, please check the log file.")

    habit_name = click.style("{0}".format(name), fg='green')
    complete_times = click.style("{0}".format(quota), fg='green')
    click.echo("Habit {0} completed {1} times".format(habit_name,complete_times))


import click
from gumshoe.classes import habits
from gumshoe.models import models

@click.command()
@click.argument("name", nargs=-1, required=True)
@click.option('--confirmation', prompt='Do you really want to remove this habit [y/n]',
              help='Confirmation that you want to remove your habit.')
def cli(name, confirmation):
    """The remove command removes a habits and all its associated activities. Please specify the name of the habit you
    wish to remove. You can use the gumshoe show command to list all habits to get the habits name."""

    name = ' '.join(name)
    db = models.HabitModel()
    new_habit = habits.Habit(name, "", "")
    habit = db.check_habit_exists(new_habit)
    if confirmation == "y":
        if len(habit) == 0:
            click.echo("This habit doe not exists, use the show command "
                       "to list all habits.")
        else:
            if db.remove_habit(habit[0]):
                click.echo("Habit removed successfully!")
            else:
                click.echo("Seems there has been a system error, please check the log file.")
    else:
        click.echo("Habit removed cancelled!")


import click
import sys
import shutil
from gumshoe.classes import habits
from gumshoe.classes import activity
from gumshoe.models import models
import logging

COMPLETE = u"\u25A0"  # 713, black square - 25A0, 25AA, 25AF
NOT_COMPLETE = u"\u25A1"  # 2717, white square - 25A1, 25AB, 25AE
PARTIAL_COMPLETE = u"\u25A0"  # t2713, green(click.style) square - 25A0, 25AA, 25AF

logger = logging.getLogger("gumshoe")

@click.command()
@click.option("-n", "--name", help="Use this option to specify the name of the habit you wish to show", default="")
@click.option("-m", "--more", help="Use this option with the -n option to show more details", is_flag=True)
@click.option("-p", "--period", help="Use this option to show all habits in a specific period, example daily, weekly or monthly", default="")
def cli(name,more,period):
    """The show commands allows you to see your progress on a specify habit as well as view all habits you have created. You can also view your current streak and longest streaks on a given habit"""
    from terminaltables import SingleTable
    from textwrap import wrap
    db = models.HabitModel()
    db2 = models.ActivityModel()
    #get terminal width
    terminal_width, terminal_height = shutil.get_terminal_size()
    #check if terminal will be big enough to display data, else give error
    nr_of_dates = terminal_width // 10 - 6
    if nr_of_dates < 1:
        logger.debug("list: Actual terminal width = {0}.".format(shutil.get_terminal_size()[0]))
        logger.debug("list: Observed terminal width = {0}.".format(terminal_width))
        click.echo("Your terminal window is too small. Please make it wider and try again")
        raise SystemExit(1)
    #if name is specified run this section
    if name != "":
        habit = db.check_habit_exists(habits.Habit(name, "", ""))
        activity_title = ""
        if len(habit) != 0:
            activities = db2.get_activities(habit[0])
            entries = activity.Activity()
            if habit[3] == "daily":
                group = entries.get_daily_activies(habit[2], activities,nr_of_dates)
                activity_title = "Daily Activities"
            elif habit[3] == "weekly":
                group = entries.get_weekly_activites(habit[2],activities,nr_of_dates)
                activity_title = "Weekly Activities"
            elif habit[3] == "monthly":
                group = entries.get_monthly_activites(habit[2],activities,nr_of_dates)
                activity_title = "Monthly Activities"
        else:
            click.echo("This habit doe not exsit, please use the show -p all command to see all available habits.")
            sys.exit()
        #if more option is specified run this options
        less = not more
        if less:
            table_title = ["Name", "Target", "Current Streak", "Longest Streak", "Period"]
            table_title.append(activity_title)
            table_rows = [table_title]
            progress = ""
            for x in group["activities"]:
                if x == "yes":
                    column_text = click.style(COMPLETE, fg="green")
                elif x == "part":
                    column_text = click.style(PARTIAL_COMPLETE)
                elif x == "no":
                    column_text = click.style(NOT_COMPLETE)
                progress += column_text + " "

            row = [str(habit[1]), habit[2], group["current_streak"], group["longest_streak"], habit[3], progress]
            table_rows.append(row)
        else:
            table_title = ["Name", "Target", "Streak", "Period"]
            row = [str(habit[1]), habit[2], group["current_streak"], habit[3]]
            for key, value in group.items():
                if key != "current_streak" and key != "longest_streak" and key != "activities":
                    table_title.append(key)
                    row.append(value)
                table_rows = [table_title]
                table_rows.append(row)
    #else if period is specified run this section
    elif period != "":
        if period != "all" and period != "daily" and period != "weekly" and period != "monthly":
            click.echo("Period - daily, weekly, or monthly, please try again")
            sys.exit()
        else:
            table_info = db.show_habit(period)
            if len(table_info) == 0:
                click.echo("There are no defined habits within this period")
                sys.exit()

            table_title = ["Name", "Target", "Periodicity"]
            table_rows = [table_title]

            for data in table_info:
                habit = data
                row = [habit[1], str(habit[2]), str(habit[3])]
                table_rows.append(row)

    else:
        click.echo("Please make sure you specify a habit name or a periodicity, use --help for more information.")
        sys.exit()

    table = SingleTable(table_rows)

    max_col_width = table.column_max_width(0)
    max_col_width = max_col_width if max_col_width > 0 else 20

    for r in table_rows:
        r[0] = '\n'.join(wrap(r[0], max_col_width))
    #print out info to screen
    click.echo(table.table)


import logging
from datetime import datetime
import sqlite3
from sqlite3 import Error
import os

database = os.path.join(os.path.dirname(__file__), "../database/gumshoe.db")
logging.basicConfig(filename=os.path.join(os.path.dirname(__file__), "../logs/app.log"), filemode='a',
                    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')


class Database:

    def create_connection(self, db_file):
        """ create a database connection to the SQLite database
            specified by db_file
        :param db_file: database file
        :return: Connection object or None
        """
        conn = None
        try:
            conn = sqlite3.connect(db_file)
            return conn
        except Error as e:
            print(e)

        return conn

    def create_table(self, conn, create_table_sql):
        """ create a table from the create_table_sql statement
        :param conn: Connection object
        :param create_table_sql: a CREATE TABLE statement
        :return:
        """
        try:
            c = conn.cursor()
            c.execute(create_table_sql)
        except Error as e:
            print(e)

    def setup(self):
        sql_create_projects_table = """ CREATE TABLE IF NOT EXISTS habits (
                                            id integer PRIMARY KEY AUTOINCREMENT,
                                            name text NOT NULL UNIQUE,
                                            quota integer NOT NULL,
                                            period text NOT NULL,
                                            created_at text NOT NULL
                                     ); """

        sql_create_tasks_table = """CREATE TABLE IF NOT EXISTS activity (
                                        id integer PRIMARY KEY AUTOINCREMENT,
                                        habit_id integer NOT NULL,
                                        created_at text NOT NULL,
                                        FOREIGN KEY (habit_id) REFERENCES habits (id)
                                    );"""

        # create a database connection
        conn = self.create_connection(database)

        # create tables
        if conn is not None:
            # create projects table
            self.create_table(conn, sql_create_projects_table)

            # create tasks table
            self.create_table(conn, sql_create_tasks_table)
        else:
            print("Error! cannot create the database connection.")


class HabitModel(Database):

    def show_habit(self,period):
        # create a database connection
        conn = self.create_connection(database)
        c = conn.cursor()
        if period == "all":
            my_cursor = c.execute("SELECT * FROM habits")
        else:
            my_cursor = c.execute("SELECT * FROM habits WHERE period = ?",(period,))
        return my_cursor.fetchall()

    def check_habit_exists(self, habit):
        try:
            conn = self.create_connection(database)
            c = conn.cursor()
            my_cursor = c.execute("SELECT * FROM habits WHERE name = ?", (habit.name,))
            result = my_cursor.fetchall()
            if not result:
                return ()
            else:
                return result[0]
        except Exception as e:
            logging.error("Exception occurred", exc_info=True)
            return ()

    def create_habit(self, habit):
        try:
            d = datetime.now()
            conn = self.create_connection(database)
            c = conn.cursor()
            c.execute("INSERT INTO habits (name, quota, period, created_at) VALUES (?, ?, ?, ?)",
                      (habit.name, habit.quota, habit.period, d.strftime("%Y-%m-%d %H:%M")))
            conn.commit()
            return True
        except Exception as e:
            logging.error("Exception occurred", exc_info=True)
            return False

    def remove_habit(self, id):
        try:
            conn = self.create_connection(database)
            c = conn.cursor()
            c.execute("DELETE FROM habits WHERE id=?", (id,))
            c.execute("DELETE FROM activity WHERE habit_id=?", (id,))
            conn.commit()
            return True
        except Exception as e:
            logging.error("Exception occurred", exc_info=True)
            return False


class ActivityModel(Database):

    def get_activities(self,habit_id):
        try:
            conn = self.create_connection(database)
            c = conn.cursor()
            my_cursor = c.execute("SELECT * FROM activity WHERE habit_id = ?", (habit_id,))
            result = my_cursor.fetchall()
            if not result:
                return ()
            else:
                return result
        except Exception as e:
            logging.error("Exception occurred", exc_info=True)
            return ()

    def complete_activity(self,habit_id):
        try:
            d = datetime.now()
            conn = self.create_connection(database)
            c = conn.cursor()
            c.execute("INSERT INTO activity (habit_id, created_at) VALUES (?, ?)",
                      (habit_id, d.strftime("%Y-%m-%d %H:%M")))
            conn.commit()
            return True
        except Exception as e:
            logging.error("Exception occurred", exc_info=True)
            return False



import sys
import click
from gumshoe.classes import habits
from gumshoe.models import models

@click.command()
@click.argument("name", nargs=-1,required=True)
@click.option("-q", "--quota", type=int, default=1, help="Use this option to increase your quota of times you need to complete a habit, default is 1")
@click.argument("period", nargs=1, required=True)
def cli(name, quota ,period):
    """The create command creates new habits and will expect the following arguments and options:\n
       name, name of the task(habit) - required\n
       period, expecting one of the following , daily, weekly, monthly - required
    """
    name = ' '.join(name)
    if period != "daily" and period != "weekly" and period != "monthly":
        click.echo("Period - daily, weekly, or monthly, please try again")
        sys.exit()

    db = models.HabitModel()
    new_habit = habits.Habit(name, quota, period)
    habit = db.check_habit_exists(new_habit)
    if len(habit) == 0:
        if db.create_habit(new_habit):
            click.echo("New habit has been created!")
        else:
            click.echo("Seems there has been a system error, please check the log file.")
    else:
        click.echo("This habit already exists.")


import sys
from datetime import datetime, timedelta
from functools import reduce
from dateutil import relativedelta


class Activity:

    def __init__(self, name="", quota="", period="", units=""):
        self.name = name
        self.quota = quota
        self.period = period
        self.units = units

    def __repr__(self):
        return "Activity('{}', '{}', {})".format(self.name, self.quota, self.period, self.units)

    def get_daily_activies(self,habit_quota,dates,nr_of_dates):
        days = {}
        activities_per_day = {}
        current_streak = 0
        longest_streak = 0
        activities = []
        streak_list = []

        for d in range(0, nr_of_dates):
            today = datetime.today()
            date_mod = today - timedelta(days=d)
            days[date_mod.strftime("%m-%d")] = []

        for date in reversed(dates):
            d = datetime.strptime(date[2], '%Y-%m-%d %H:%S')
            if d.month > 9:
                s = ""
            else:
                s = "0"
            if d.day > 9:
                f = ""
            else:
                f = "0"
            if s + str(d.month) + "-" + f + str(d.day) in days:
                get_list = days[s + str(d.month) + "-" + f + str(d.day)]
                get_list.append(date)
            else:
                days[s + str(d.month) + "-" + f + str(d.day)] = [date]

        for key, value in days.items():
            if reduce(lambda x, y: x + 1, value, 0) > 0:
                activities_per_day[key] = str(reduce(lambda x, y: x + 1, value, 0))
            else:
                activities_per_day[key] = 0

        for key, value in activities_per_day.items():
            streak_list.append(value)
            if int(value) >= int(habit_quota):
                activities.append("yes")
            elif int(value) > 0:
                activities.append("part")
            else:
                activities.append("no")

        for x in reversed(streak_list):
            if int(x) >= int(habit_quota):
                current_streak += 1
            else:
                if longest_streak == 0:
                    longest_streak = current_streak
                elif current_streak > longest_streak:
                    longest_streak = current_streak
                current_streak = 0

            if current_streak > longest_streak:
                longest_streak = current_streak

        activities_per_day["current_streak"] = current_streak
        activities_per_day["longest_streak"] = longest_streak
        activities_per_day["activities"] = activities

        return activities_per_day

    def get_weekly_activites(self,habit_quota,dates,nr_of_dates):
        weeks = {}
        activities_per_week = {}
        current_streak = 0
        longest_streak = 0
        activities = []
        streak_list = []

        for d in range(0, nr_of_dates):
            today = datetime.today()
            last_sunday_offset = today.weekday()  # convert day format mon-sun=0-6 => sun-sat=0-6
            last_sunday = today - timedelta(days=last_sunday_offset)
            date_mod = last_sunday - timedelta(weeks=d)
            weeks[date_mod.strftime("%y-%m-%d")] = []


        for date in reversed(dates):
            d1 = datetime.strptime(date[2], '%Y-%m-%d %H:%S')
            for key, value in weeks.items():
                d2 = datetime.strptime(key, '%y-%m-%d')
                if d1.isocalendar()[1] == d2.isocalendar()[1]:
                    get_list = weeks[d2.strftime("%y-%m-%d")]
                    get_list.append(date)

        for key, value in weeks.items():
            if reduce(lambda x, y: x + 1, value, 0) > 0:
                activities_per_week[key] = str(reduce(lambda x, y: x + 1, value, 0))
            else:
                activities_per_week[key] = 0

        for key, value in activities_per_week.items():
            streak_list.append(value)
            if int(value)>=int(habit_quota):
                activities.append("yes")
            elif int(value) > 0:
                activities.append("part")
            else:
                activities.append("no")

        for x in reversed(streak_list):
            if int(x) >= int(habit_quota):
                current_streak += 1
            else:
                if longest_streak == 0:
                    longest_streak = current_streak
                elif current_streak > longest_streak:
                    longest_streak = current_streak
                current_streak = 0

            if current_streak > longest_streak:
                longest_streak = current_streak

        activities_per_week["current_streak"] = current_streak
        activities_per_week["longest_streak"] = longest_streak
        activities_per_week["activities"] = activities

        return activities_per_week

    def get_monthly_activites(self,habit_quota,dates,nr_of_dates):
        months = {}
        activities_per_month = {}
        current_streak = 0
        longest_streak = 0
        activities = []
        streak_list = []

        for d in range(0, nr_of_dates):
            today = datetime.today()
            a_month = relativedelta.relativedelta(months=d)
            date_minus_month = today - a_month
            months[date_minus_month.strftime("%Y-%m")] = []

        for date in reversed(dates):
            d = datetime.strptime(date[2], '%Y-%m-%d %H:%S')
            if d.month > 9:
                s = ""
            else:
                s = "0"
            if str(d.year) + "-"+s+str(d.month) in months:
                get_list = months[str(d.year) + "-"+s+str(d.month)]
                get_list.append(date)
            else:
                months[str(d.year) + "-"+s+str(d.month)] = [date]

        for key, value in months.items():
            if reduce(lambda x, y: x + 1, value, 0) > 0:
                activities_per_month[key] = str(reduce(lambda x, y: x + 1, value, 0))
            else:
                activities_per_month[key] = 0

        for key, value in activities_per_month.items():
            streak_list.append(value)
            if int(value)>=int(habit_quota):
                activities.append("yes")
            elif int(value) > 0:
                activities.append("part")
            else:
                activities.append("no")

        for x in reversed(streak_list):
            if int(x) >= int(habit_quota):
                current_streak += 1
            else:
                if longest_streak == 0:
                    longest_streak = current_streak
                elif current_streak > longest_streak:
                    longest_streak = current_streak
                current_streak = 0

            if current_streak > longest_streak:
                longest_streak = current_streak


        activities_per_month["current_streak"] = current_streak
        activities_per_month["longest_streak"] = longest_streak
        activities_per_month["activities"] = activities

        return activities_per_month





from gumshoe.models import models
from gumshoe.classes import gumshoecli

db = models.Database()
db.setup()

cli = gumshoecli.GumShoeCli(help='Welcome to GumShoe the habit tracking command line tool.')

if __name__ == "__main__":
    cli()



class Habit:

    def __init__(self, name, quota , period):
        self.name = name
        self.quota = quota
        self.period = period

    def __repr__(self):
        return "Habit('{}', '{}', {})".format(self.name, self.quota, self.period)


"""Setup for Habito app."""

from setuptools import setup, find_packages


setup(
    name="gumshoe-cli",
    version="1.1.6",
    description="Your habit tracking command line tool.",
    long_description="Your habit tracking command line tool.",
    url="https://github.com/philipcsaplar/gumshoe-cli/tree/master/gumshoe",
    author="Philip Csaplar",
    author_email="philip@osit.co.za",

    packages=find_packages(exclude=["tests*"]),
    entry_points={
        "console_scripts": ["gumshoe=gumshoe.main:cli"],
    },
    install_requires=["click", "terminaltables"],

    license="MIT",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS",
        "Operating System :: Microsoft",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Utilities",
    ],
    keywords="habits goals track tracking quantified self",
)
